@@ -77,6 +77,7 @@ class Macro:
 
             self.setup_commands += [(x, None) for x in [
                 SetupHome(),
+                MakeNonInteractive(),
                 InitSetupRepos(),
                 repo_loop,
                 FinishSetupRepos(),
@@ -273,6 +274,7 @@ class SetupHome(Command):
         aws_dir = self.tmpdirname + "/.aws"
         conf_file = aws_dir + "/config"
         shared_creds_file = aws_dir + "/credentials"
+        sso_cache_dir = aws_dir + "/sso/cache"
         os.makedirs(aws_dir)
         aws_conf_file = self._path_from_env('AWS_CONFIG_FILE')
         aws_shared_creds_file = \
@@ -297,6 +299,26 @@ class SetupHome(Command):
                 config.write(fds)
             shutil.copy(aws_web_identity_token_file, webid_token_file)
 
+        # SSO workflow
+        if aws_conf_file:
+            aws_cache_dir_conf = \
+                os.path.join(os.path.dirname(aws_conf_file), "sso/cache")
+            aws_cache_dir_home = os.path.join(Path.home(), ".aws/sso/cache")
+
+            # In kas-container the directory passed in --aws-dir is not
+            # mounted in $HOME. Look for sso/cache in directory containing
+            # AWS_CONFIG_FILE first to maintain the same behavior between
+            # kas and kas-container.
+            aws_cache_dir = None
+            if os.path.isdir(aws_cache_dir_conf):
+                aws_cache_dir = aws_cache_dir_conf
+            elif os.path.isdir(aws_cache_dir_home):
+                aws_cache_dir = aws_cache_dir_home
+
+            if aws_cache_dir:
+                shutil.copy(aws_conf_file, conf_file)
+                shutil.copytree(aws_cache_dir, sso_cache_dir)
+
     @staticmethod
     def _setup_gitlab_ci_ssh_rewrite(config):
         ci_host = os.environ.get('CI_SERVER_HOST', None)
@@ -372,6 +394,30 @@ class SetupHome(Command):
         ctx.environ['HOME'] = self.tmpdirname
 
 
+class MakeNonInteractive(Command):
+    """
+        Make execution environment non-interactive
+    """
+
+    def __str__(self):
+        return 'make_non_interactive'
+
+    def execute(self, ctx):
+        ctx.environ['GIT_TERMINAL_PROMPT'] = 'false'
+
+
+class MakeInteractive(Command):
+    """
+        Make execution environment interactive
+    """
+
+    def __str__(self):
+        return 'make_interactive'
+
+    def execute(self, ctx):
+        ctx.environ['GIT_TERMINAL_PROMPT'] = 'true'
+
+
 class SetupDir(Command):
     """
         Creates the build directory.
@@ -443,21 +489,23 @@ class WriteBBConfig(Command):
                 It is not expanded by KAS, hence we avoid
                 absolute paths pointing into the build host.
             """
-            relpath = os.path.relpath(layer, ctx.build_dir)
+            # we need to walk up, which would require pathlib.Path >= 3.12
+            relpath = os.path.relpath(layer.path, ctx.build_dir)
             return '${TOPDIR}/' + relpath
 
         def _write_bblayers_conf(ctx):
             filename = ctx.build_dir + '/conf/bblayers.conf'
             if not os.path.isdir(os.path.dirname(filename)):
                 os.makedirs(os.path.dirname(filename))
+            layers_sorted = \
+                sorted([layer for repo in ctx.config.get_repos()
+                        for layer in repo.layers])
             with open(filename, 'w') as fds:
                 fds.write(ctx.config.get_bblayers_conf_header())
                 fds.write('BBLAYERS ?= " \\\n    ')
                 fds.write(' \\\n    '.join(
                           [_get_layer_path_under_topdir(ctx, layer)
-                           for repo in sorted(ctx.config.get_repos(),
-                                              key=lambda r: r.name)
-                           for layer in sorted(repo.layers)]))
+                           for layer in layers_sorted]))
                 fds.write('"\n')
                 fds.write('BBPATH ?= "${TOPDIR}"\n')
                 fds.write('BBFILES ??= ""\n')
