@@ -81,7 +81,7 @@ from kas.includehandler import ConfigFile
 from kas.plugins.checkout import Checkout
 from kas.plugins.dump import Dump, IoTarget, LOCKFILE_VERSION_MIN
 from kas.plugins.diff import Diff
-from kas.repos import Repo
+from kas.repos import Repo, RepoRefError
 
 __license__ = 'MIT'
 __copyright__ = 'Copyright (c) Siemens AG, 2024'
@@ -112,6 +112,9 @@ class Lock(Checkout):
             diff = repo.diff(old_commit, None)
         except NotImplementedError:
             return
+        except RepoRefError as e:
+            logging.warning(e)
+            return
         Diff.formatting_diff_output(
             None, None, {'vcs': diff}, True, False, True, False)
 
@@ -141,9 +144,10 @@ class Lock(Checkout):
                 continue
 
             r = ri.repo
-            if v['commit'] == r.revision:
+            if v['commit'] == r.revision or \
+                    v['commit'] == r.revision_annotated_tag:
                 logging.info('Lock of %s is up-to-date: %s',
-                             r.name, r.revision)
+                             r.name, v['commit'])
             elif not lockfile.is_external:
                 logging.info('Updating lock of %s: %s -> %s',
                              r.name, v['commit'], r.revision)
