@@ -28,14 +28,18 @@ import os
 import linecache
 import logging
 import shutil
+from pathlib import Path
+from dataclasses import dataclass
 from datetime import datetime
 from urllib.parse import urlparse
 from tempfile import TemporaryDirectory
+
+from kas.configschema import CONFIGSCHEMA
 from .context import get_context
 from .libkas import run_cmd_async, run_cmd
 from .kasusererror import KasUserError
 from functools import cached_property
-from git import Repo as GitPythonRepo
+from git import GitCommandError, Repo as GitPythonRepo
 
 __license__ = 'MIT'
 __copyright__ = 'Copyright (c) Siemens AG, 2017-2018'
@@ -105,7 +109,7 @@ class Repo:
         self.tag = tag
         self.branch = branch
         self.refspec = refspec
-        self._layers = layers
+        self.layers = layers
         self._patches = patches
         self.allowed_signers = signers
         self.operations_disabled = disable_operations
@@ -113,11 +117,6 @@ class Repo:
         if not self.url:
             self.resolve_local()
 
-    @property
-    def layers(self):
-        return [os.path.join(self.path, layer).rstrip(os.sep + '.')
-                for layer in self._layers]
-
     @property
     def qualified_name(self):
         url = urlparse(self.url)
@@ -162,6 +161,19 @@ class Repo:
             return branch
         return None
 
+    @cached_property
+    def revision_annotated_tag(self):
+        if not self.tag:
+            return None
+        try:
+            cmd = self.resolve_annotated_tag_cmd()
+        except NotImplementedError:
+            return None
+        (retc, output) = run_cmd(cmd, cwd=self.path, fail=False)
+        if retc or not output:
+            return None
+        return output.strip()
+
     @cached_property
     def dirty(self):
         if not self.url:
@@ -224,7 +236,7 @@ class Repo:
             refspec = self.commit or self.tag or self.branch or self.refspec
 
         return f'{self.url}:{refspec} ' \
-               f'{self.path} {self._layers}'
+               f'{self.path} {self.layers}'
 
     __legacy_refspec_warned__ = []
     __no_commit_tag_warned__ = []
@@ -236,18 +248,6 @@ class Repo:
             Returns a Repo instance depending on parameters.
             This factory function is referential transparent.
         """
-        layers_dict = repo_config.get('layers', {'': None})
-        # only bool(false) will be a valid value to disable a layer
-        for lname, prop in layers_dict.items():
-            if not (prop is None or prop == "disabled"):
-                logging.warning('Use of deprecated value "%s" for repo '
-                                '"%s", layer "%s". Replace with "disabled".',
-                                prop, name, lname)
-
-        layers = list(filter(lambda x, laydict=layers_dict:
-                             str(laydict[x]).lower() not in
-                             ['disabled', 'excluded', 'n', 'no', '0', 'false'],
-                             layers_dict))
         default_patch_repo = repo_defaults.get('patches', {}).get('repo', None)
         patches_dict = repo_config.get('patches', {})
         patches = []
@@ -319,6 +319,9 @@ class Repo:
             # Relative pathes are assumed to start from work_dir
             path = os.path.join(get_context().kas_work_dir, path)
 
+        layers_dict = repo_config.get('layers', {'': None})
+        layers = Repo._create_layers_from_dict(name, path, layers_dict)
+
         if url is None:
             # No version control operation on repository
             disable_operations = True
@@ -359,6 +362,38 @@ class Repo:
 
         return path if fallback else None
 
+    @staticmethod
+    def _create_layers_from_dict(repo_name, repo_path, layers_dict):
+        layers = []
+        disabled_token = "disabled"
+        legacy_disabled_tokens = ['excluded', 'n', 'no', '0', 'false', 0]
+        default_prio = \
+            CONFIGSCHEMA['$defs']['layerPrio']['properties']['prio']['default']
+
+        for lname, prop in layers_dict.items():
+            if prop is None:
+                layers.append(RepoLayer(name=lname,
+                                        repo_path=Path(repo_path),
+                                        repo_name=repo_name,
+                                        priority=default_prio))
+            elif isinstance(prop, str) and prop == disabled_token:
+                continue
+            elif isinstance(prop, (str, int)) and \
+                    prop in legacy_disabled_tokens:
+                logging.warning('Use of deprecated value "%s" for repo '
+                                '"%s", layer "%s". Replace with "disabled".',
+                                prop, repo_name, lname)
+                continue
+            elif isinstance(prop, dict):
+                layers.append(RepoLayer(name=lname,
+                                        repo_path=Path(repo_path),
+                                        repo_name=repo_name,
+                                        priority=prop.get('prio',
+                                                          default_prio)))
+            else:
+                raise NotImplementedError()
+        return layers
+
 
 class RepoImpl(Repo):
     """
@@ -441,13 +476,11 @@ class RepoImpl(Repo):
 
         # Try to fetch if commit/tag/branch/refspec is missing or if --update
         # argument was passed
-        (retc, output) = await run_cmd_async(self.fetch_cmd(),
+        (retc, _, err) = await run_cmd_async(self.fetch_cmd(),
                                              cwd=self.path,
-                                             fail=False)
+                                             fail=False, capture_stderr=True)
         if retc:
-            logging.error('Could not update repository %s: %s',
-                          self.name, output)
-            raise RepoFetchError(self, output)
+            raise RepoFetchError(self, err)
         else:
             logging.info('Repository %s updated', self.name)
 
@@ -481,11 +514,12 @@ class RepoImpl(Repo):
 
             desired_ref = output.strip()
             if self.commit and desired_ref != self.commit:
-                # Ensure provided commit and tag match
-                raise RepoRefError(f'Provided tag "{self.tag}" '
-                                   f'("{desired_ref}") does not match '
-                                   f'provided commit "{self.commit}" in '
-                                   f'repository "{self.name}", aborting!')
+                if self.revision_annotated_tag != self.commit:
+                    # Ensure provided commit and tag match
+                    raise RepoRefError(f'Provided tag "{self.tag}" '
+                                       f'("{desired_ref}") does not match '
+                                       f'provided commit "{self.commit}" in '
+                                       f'repository "{self.name}", aborting!')
             is_branch = False
         elif self.branch:
             (retc, output) = run_cmd(self.resolve_branch_cmd(),
@@ -718,6 +752,9 @@ class GitRepo(RepoImpl):
     def resolve_tag_cmd(self):
         return ['git', 'rev-list', '-n', '1', self.remove_ref_prefix(self.tag)]
 
+    def resolve_annotated_tag_cmd(self):
+        return ['git', 'show-ref', '--tags', '--hash', self.tag]
+
     def branch_contains_ref(self):
         return ['git', 'branch', f'origin/{self.branch}',
                 '-r', '--contains', self.commit]
@@ -768,8 +805,23 @@ class GitRepo(RepoImpl):
         shallow_file = os.path.join(git_repo.git_dir, 'shallow')
         if os.path.isfile(shallow_file):
             git_repo.git.fetch(unshallow=True)
-        commits = list(git_repo.iter_commits(
-                       f'{commit1}..{commit2}'))
+        try:
+            commits = list(git_repo.iter_commits(
+                           f'{commit1}..{commit2}'))
+        except GitCommandError as e:
+            valid1 = git_repo.is_valid_object(commit1)
+            valid2 = git_repo.is_valid_object(commit2)
+            if not valid1 and not valid2:
+                error_msg = f'Commits ({commit1} and {commit2}) are not valid'
+            elif not valid1:
+                error_msg = f'First commit ({commit1}) is not valid'
+            elif not valid2:
+                error_msg = f'Second commit ({commit2}) is not valid'
+            else:
+                error_msg = str(e)
+            raise RepoRefError(
+                'Could not compute diff for repository '
+                f'"{self.name}": {error_msg}')
         diff_json = {self.name: []}
         for commit in commits:
             diff_json[self.name].append({
@@ -829,6 +881,9 @@ class MercurialRepo(RepoImpl):
         refspec = self.tag or self.refspec
         return ['hg', 'identify', '--id', '-r', f'tag({refspec})']
 
+    def resolve_annotated_tag_cmd(self):
+        raise NotImplementedError("Mercurial does not support annotated tags")
+
     def branch_contains_ref(self):
         return ['hg', 'log', '-r', self.commit, '-b', self.branch]
 
@@ -871,3 +926,26 @@ class MercurialRepo(RepoImpl):
 
     def diff(self, commit1, commit2):
         raise NotImplementedError("Unsupported diff for MercurialRepo")
+
+
+@dataclass
+class RepoLayer:
+    """
+    Standalone definition of a single bitbake layer.
+    """
+    name: str
+    priority: int
+    repo_name: str
+    repo_path: Path
+
+    @property
+    def path(self):
+        return self.repo_path / self.name
+
+    def __lt__(self, other):
+        if isinstance(other, RepoLayer):
+            # The priority is negated because a higher priority means the
+            # element appears earlier in the list.
+            return (-self.priority, self.repo_name, self.name) < \
+                (-other.priority, other.repo_name, other.name)
+        return NotImplemented
